## Global Utils
library(tiff)
library(data.table)
library(ggplot2)
library(scales)
library(imgnoiser)

soften <- function(mask, df=20) {
  sp <- smooth.spline(x=src, y=mask, all.knots = TRUE)
  sp <- smooth.spline(x=src, y=mask, df=min(sp$df*0.60, df), all.knots = TRUE)
  # Result
  fit <- predict(sp, src);
  fit <- fit$y
  #   browser()
  below.zero <- which(fit < 0)
  if (any(below.zero))  fit[below.zero] <- mask[below.zero]
  fit;
}

mask_type <- NULL
data.table.mask.type <- function(mask.type) {
  mask_type <<- mask.type
}

mask.as.data.table.generic <- function(alpha, name, df=20, y.scale=NULL, x.is.Lab=FALSE, lab.from.src) {

  normal.y <- if (is.null(y.scale) || y.scale == 1) 'both' else 'no'
  result <- rbindlist(use.names=TRUE,
            list(
              data.table('tone'=src, 'name' = name, 'alpha' = soften(alpha, df), 'normal.y' = normal.y, 'x.is.lab' = FALSE)
              ,data.table('tone'=lab.from.src(src), 'name' = name, 'alpha' = soften(alpha, df), 'normal.y' = normal.y, 'x.is.lab' = TRUE)
            ))

  if (!is.null(y.scale) && y.scale != 1) {
    normal.y <- 'yes'
    result <- rbindlist(use.names=TRUE,
                        list(
                          result
                          ,data.table('tone'=src, 'name' = name, 'alpha' = soften(alpha, df)*y.scale, 'normal.y' = normal.y, 'x.is.lab' = FALSE)
                          ,data.table('tone'=lab.from.src(src), 'name' = name, 'alpha' = soften(alpha, df)*y.scale, 'normal.y' = normal.y, 'x.is.lab' = TRUE)
                        ))

  }

  # Result
  cbind(data.table('type' = mask_type), result);
}

mask.as.data.table.pf <- function(alpha, name, df=20, y.scale=NULL, x.is.Lab=FALSE) {
  mask.as.data.table.generic(alpha, name, df, y.scale, x.is.Lab, lab.from.src=lab.from.proPhoto);
}

mask.as.data.table.sRGB <- function(alpha, name, df=20, y.scale=NULL, x.is.Lab=FALSE) {
  mask.as.data.table.generic(alpha, name, df, y.scale, x.is.Lab, lab.from.src=L.Lab.from.sRGB);
}

mask.as.data.table.pf2.2 <- function(alpha, name, df=20, y.scale=NULL, x.is.Lab=FALSE) {
  mask.as.data.table.generic(alpha, name, df, y.scale, x.is.Lab, lab.from.src=lab.from.proPhoto2.2);
}

mask.as.data.table.aRGB <- function(alpha, name, df=20, y.scale=NULL, x.is.Lab=FALSE) {
  mask.as.data.table.generic(alpha, name, df, y.scale, x.is.Lab, lab.from.src=L.Lab.from.adobeRGB);
}

label.pow10.percent <- function(x) {
  marks <- 10^x*100
  marks <- round(marks, 1)
  paste0(marks,'%');
}

label.pow10.plus.percent <- function(x) {
  marks <- 10^x
  marks <- round(marks, 1)
  paste0(marks,'%');
}

label.percent.suffix <- function(x) {
  marks <- round(x, 1)
  paste0(marks,'%');
}

lab.label <- function(lab) {
  lab.tones <- round(lab,1)
  paste0(lab.tones,'L');
}

lab.from.proPhoto <- function(proPhoto) {
  L.Lab.from.linear(linear.from.proPhoto(proPhoto));
}

lab.from.proPhoto2.2 <- function(proPhoto2.2) {
  L.Lab.from.linear(proPhoto2.2^2.2);
}

lab.label.from.prophoto <- function(proPhoto) {
  lab <- lab.from.proPhoto(proPhoto)
  lab <- round(lab,1)
  # Return
  paste(lab,'L');
}

lab.label.from.sRGB <- function(sRGB) {
  lab <- L.Lab.from.sRGB(sRGB)
  lab <- round(lab,1)
  # Return
  paste(lab,'L');
}

lab.label.from.prophoto2.2 <- function(proPhoto2.2) {
  lab <- lab.from.proPhoto2.2(proPhoto2.2)
  lab <- round(lab,1)
  # Return
  paste(lab,'L');
}

lab.label.from.adobeRGB <- function(x) {
  lab.tones <- L.Lab.from.adobeRGB(x)
  lab.tones <- round(lab.tones,1)
  paste(lab.tones,'L');
}

plot.generic.mask <- function(mask.data,
                         mask.type='',
                         show.lab.x = FALSE,
                         plot.lab.x = FALSE,
                         normal.scale = FALSE,
                         space.name,
                         lab.from.src
                ) {

  normal.y.label <-
    if (!normal.scale) 'yes' else 'no';

  if (plot.lab.x) {
    x.breaks <- seq(0,100,10)
    x.labels <- lab.label
    show.lab.x <- TRUE
  } else {
    x.breaks <- seq(0,1,0.1)
    x.labels <-
      if (show.lab.x) lab.from.src
      else percent
  }

  title <- paste('Masks in', space.name, 'Space')
  xlabel <- paste(space.name, 'Tonal scale')
  title <-
    if (mask.type != '') paste(mask.type, title)
    else title

  if (normal.scale == TRUE)
    title <- paste(title,'\nWith normalized y Scale')
  else
    title <- paste(title,'\n')

  ggplot(subset(mask.data, x.is.lab == plot.lab.x & normal.y != normal.y.label),
         aes(tone, alpha, group=name, color=name, linetype=name)) +
    ggtitle(title) +
#     labs(linetype='Mask Name') +
    xlab(ifelse(plot.lab.x, 'L (Lab) Tonal Scale', xlabel)) +
    ylab('Mask Transparency') +
    geom_line() +
    scale_x_continuous(breaks = x.breaks, labels = x.labels) +
    scale_y_continuous(limits=c(0,1), breaks=seq(0,1,0.1), label=percent)
}

plot.pf.mask <- function(mask.data,
                         mask.type = '',
                         show.lab.x = FALSE,
                         plot.lab.x = FALSE,
                         normal.scale = FALSE ) {

  plot.generic.mask(mask.data,
                    mask.type,
                    show.lab.x,
                    plot.lab.x,
                    normal.scale,
                    space.name = 'ProPhoto',
                    lab.from.src = lab.label.from.prophoto
                    )
}

plot.pf2.2.mask <- function(mask.data,
                         mask.type = '',
                         show.lab.x = FALSE,
                         plot.lab.x = FALSE,
                         normal.scale = FALSE ) {

  plot.generic.mask(mask.data,
                    mask.type,
                    show.lab.x,
                    plot.lab.x,
                    normal.scale,
                    space.name = 'ProPhoto Gamma 2.2',
                    lab.from.src = lab.label.from.prophoto2.2
  )
}

plot.pfsRGB.mask <- function(mask.data,
                            mask.type = '',
                            show.lab.x = FALSE,
                            plot.lab.x = FALSE,
                            normal.scale = FALSE ) {

  plot.generic.mask(mask.data,
                    mask.type,
                    show.lab.x,
                    plot.lab.x,
                    normal.scale,
                    space.name = 'Melissa RGB',
                    lab.from.src = lab.label.from.sRGB
  )
}

plot.aRGB.mask <- function(mask.data,
                         mask.type = '',
                         show.lab.x = FALSE,
                         plot.lab.x = FALSE,
                         normal.scale = FALSE ) {

  plot.generic.mask(mask.data,
                    mask.type,
                    show.lab.x,
                    plot.lab.x,
                    normal.scale,
                    space.name = 'Adobe RGB',
                    lab.from.src = lab.label.from.adobeRGB
  )
}