# Adobe RGB with gray gamma 2.2 (natural best)

source('Utils.R')

lmask1t3 <- readTIFF('lumask-l1-l3.tif')
src <- lmask1t3[1,,1]
# Checking Unicity
unique.value.index <- as.data.table(src)[, list(list(.I)), by = src]


### Light Masks

lmask1t3 <- readTIFF('lumask-l1-l3.tif')
lmask4t6 <- readTIFF('lumask-l4-l6.tif')
lmaskl1q <- readTIFF('lumask-l1q.tif')
lmaskl1q.odl <- readTIFF('lumask-l1q-odl.tif')

data.table.mask.type('Lights')
adobeRGB.lmasks <-
  rbindlist(use.names=TRUE,
            list(
              mask.as.data.table.aRGB(lmask1t3[1,,1], '(1) Lights'),
              mask.as.data.table.aRGB(lmask1t3[1,,2], '(2) Lights Lights'),
              mask.as.data.table.aRGB(lmask1t3[1,,3], '(3) Bright Lights'),
              mask.as.data.table.aRGB(lmask4t6[1,,1], '(4) Super Lights'),
              mask.as.data.table.aRGB(lmask4t6[1,,2], '(5) Ultra Lights'),
              mask.as.data.table.aRGB(lmask4t6[1,,3], '(6) Mid-tone Lights (ML)', y.scale=1.335573),
              mask.as.data.table.aRGB(lmaskl1q[1,,1], '(7) 1/4 Lights (1/4)', y.scale=1.459237)
             ,mask.as.data.table.aRGB(lmaskl1q.odl[1,,1], '(8+) 1/4 Mid-Lights', y.scale=1.3586)
  ))

x11()
plot.aRGB.mask(
  adobeRGB.lmasks, mask.type='Lights',
  show.lab.x = FALSE,
  plot.lab.x = FALSE,
  normal.scale = FALSE
)

### Dark Masks

dmask1t3 <- readTIFF('lumask-d1-d3.tif')
dmask4t6 <- readTIFF('lumask-d4-d6.tif')
dmaskl3q <- readTIFF('lumask-d3q.tif')

data.table.mask.type('Darks')
adobeRGB.dmasks <-
  rbindlist(use.names=TRUE,
           list(
             mask.as.data.table.aRGB(dmask1t3[1,,1], '(1) Darks'),
             mask.as.data.table.aRGB(dmask1t3[1,,2], '(2) Dark Darks'),
             mask.as.data.table.aRGB(dmask1t3[1,,3], '(3) Shadow Darks'),
             mask.as.data.table.aRGB(dmask4t6[1,,1], '(4) Super Darks'),
             mask.as.data.table.aRGB(dmask4t6[1,,2], '(5) Ultra Darks'),
             mask.as.data.table.aRGB(dmask4t6[1,,3], '(6) Mid-tone Darks (MD)', y.scale = 1.335573),
             mask.as.data.table.aRGB(dmaskl3q[1,,1], '(7) 3/4 Darks (3/4)', y.scale = 1.452233)
           ))

x11()
plot.aRGB.mask(
  adobeRGB.dmasks, mask.type='Darks',
  show.lab.x = FALSE,
  plot.lab.x = FALSE,
  normal.scale = TRUE
)

### Mid Tone Masks

mmask1t3 <- readTIFF('lumask-m1-m3.tif')
mmask4t5 <- readTIFF('lumask-m4-m5.tif')
mmaskodl <- readTIFF('lumask-m-odl.tif')
mmaskodl.v2 <- readTIFF('lumask-m-odl-v2.tif')
mmaskodl.v3 <- readTIFF('lumask-m-odl-v3.tif')

data.table.mask.type('Mid')
adobeRGB.mmasks <-
  rbindlist(use.names=TRUE,
           list(
             mask.as.data.table.aRGB(mmask1t3[1,,1], '(1) Basic Mid-tones', y.scale = 2.462463)
              ,mask.as.data.table.aRGB(mmask1t3[1,,2], '(2) Expanded Mid-tones', y.scale = 1.770881)
              ,mask.as.data.table.aRGB(mmask1t3[1,,3], '(3) Wide Mid-tones', y.scale = 1.138413)
             ,mask.as.data.table.aRGB(mmask4t5[1,,1], '(4) Super Mid-tones', y.scale = 1)
             ,mask.as.data.table.aRGB(mmask4t5[1,,2], '(5) Ultra Mid-tones', y.scale = 1)

              ,mask.as.data.table.aRGB(mmaskodl.v2[1,,1], '(6+) Inner Mid-tones', y.scale = 1.876162)
              ,mask.as.data.table.aRGB(mmaskodl[1,,1], '(7+) Mid Mid-tones', y.scale = 0.8208377)
           ))

x11()
plot.aRGB.mask(
  adobeRGB.mmasks, mask.type='Mid-tones',
  show.lab.x = FALSE,
  plot.lab.x = FALSE,
  normal.scale = TRUE
)

### Zone Masks

aRgb0t2 <- readTIFF('aRgb0t2.tif')
aRgb3t5 <- readTIFF('aRgb3t5.tif')
aRgb6t8 <- readTIFF('aRgb6t8.tif')
aRgb9tX <- readTIFF('aRgb9tX.tif')

data.table.mask.type('Zone')
adobeRGB.zmasks <-
  rbindlist(use.names=TRUE,
            list(
              mask.as.data.table.aRGB(aRgb0t2[1,,1], 'Zone 0', y.scale = 1),
              mask.as.data.table.aRGB(aRgb0t2[1,,2], 'Zone 1', y.scale = 1.608139),
              mask.as.data.table.aRGB(aRgb0t2[1,,3], 'Zone 2', y.scale = 1.608139),
              mask.as.data.table.aRGB(aRgb3t5[1,,1], 'Zone 3', y.scale = 1.608139),
              mask.as.data.table.aRGB(aRgb3t5[1,,2], 'Zone 4', y.scale = 1.608139),
              mask.as.data.table.aRGB(aRgb3t5[1,,3], 'Zone 5', y.scale = 2.462463),
              mask.as.data.table.aRGB(aRgb6t8[1,,1], 'Zone 6', y.scale = 1.608139),
              mask.as.data.table.aRGB(aRgb6t8[1,,2], 'Zone 7', y.scale = 1.608139),
              mask.as.data.table.aRGB(aRgb6t8[1,,3], 'Zone 8', y.scale = 1.608139),
              mask.as.data.table.aRGB(aRgb9tX[1,,1], 'Zone 9', y.scale = 1.608139),
              mask.as.data.table.aRGB(aRgb9tX[1,,2], 'Zone X', y.scale = 1)
            ))


x11()
plot.aRGB.mask(
  adobeRGB.zmasks, mask.type='Zone',
  show.lab.x = TRUE,
  plot.lab.x = TRUE,
  normal.scale = TRUE
)

### Regression analysis

lights.df <- data.frame('x' = src, 'y' = lmask1t3[1,,3])
lights.df <- subset(lights.df, x > 0 & y > 0)

lr <- lm(log(y) ~ log(x), data=lights.df)
summary(lr)

lights.df$y.est <- lights.df$x^2
qplot(y, y.est/y, data=lights.df)

### Export Data

adobeRGB.all_masks <-
  rbindlist(use.names=TRUE,
            list(
              adobeRGB.lmasks,
              adobeRGB.dmasks,
              adobeRGB.mmasks,
              adobeRGB.zmasks
            ))

write.csv(adobeRGB.all_masks, 'adobeRGB-all-masks.csv')
