## Pro Photo with gray gamma 1.8
source('Utils.R')

# Checking Unicity
lpf1t3 <- readTIFF('lpf1t3.tif')
src <- lpf1t3[1,,1]
unique.value.index <- as.data.table(src)[, list(list(.I)), by = src]


### Light Masks

lpf1t3 <- readTIFF('lpf1t3.tif')
lpf4t6 <- readTIFF('lpf4t6.tif')
lpf1q <- readTIFF('lpf1q.tif')
lpf1q.odl <- readTIFF('lpf1q-odl.tif')

data.table.mask.type('Light')
proPhoto.lmasks <-
  rbindlist(use.names=TRUE,
           list(
             mask.as.data.table.pf(lpf1t3[1,,1], '(1) Lights'),
             mask.as.data.table.pf(lpf1t3[1,,2], '(2) Lights Lights'),
             mask.as.data.table.pf(lpf1t3[1,,3], '(3) Bright Lights'),
             mask.as.data.table.pf(lpf4t6[1,,1], '(4) Super Lights'),
             mask.as.data.table.pf(lpf4t6[1,,2], '(5) Ultra Lights'),
             mask.as.data.table.pf(lpf4t6[1,,3], '(6) Mid-tone Lights (ML)', y.scale = 1.335573),
             mask.as.data.table.pf(lpf1q[1,,1], '(7) 1/4 Lights (1/4)', y.scale = 1.452233)
#              .mask.as.data.table.pf(lpf1q.odl[1,,1], '(8+) 1/4 Mid-Lights', y.scale = 1.3586)
           ))

x11()
plot.pf.mask(
    proPhoto.lmasks, mask.type='Light',
    show.lab.x = TRUE,
    plot.lab.x = TRUE,
    normal.scale = TRUE
  )


### Dark Masks

dpf1t3 <- readTIFF('dpf1t3.tif')
dpf4t6 <- readTIFF('dpf4t6.tif')
dpf3q.odl <- readTIFF('dpf3q-odl.tif')

data.table.mask.type('Dark')
proPhoto.dmasks <-
  rbindlist(use.names=TRUE,
           list(
             mask.as.data.table.pf(dpf1t3[1,,1], '(1) Darks'),
             mask.as.data.table.pf(dpf1t3[1,,2], '(2) Dark Darks'),
             mask.as.data.table.pf(dpf1t3[1,,3], '(3) Shadow Darks'),
             mask.as.data.table.pf(dpf4t6[1,,1], '(4) Super Darks'),
             mask.as.data.table.pf(dpf4t6[1,,2], '(5) Ultra Darks'),
             mask.as.data.table.pf(dpf4t6[1,,3], '(6) Mid-tone Darks (MD)', y.scale = 1.335573),
             mask.as.data.table.pf(dpf3q[1,,1], '(7) 3/4 Darks (3/4)', y.scale = 1.452233),
             mask.as.data.table.pf(dpf3q.odl[1,,1], '(8+) 3/4 Mid-Darks', y.scale = 1.3586)
           ))

x11()
plot.pf.mask(
  proPhoto.dmasks, mask.type='Dark',
  show.lab.x = FALSE,
  plot.lab.x = TRUE,
  normal.scale = TRUE
)

### Mid Tone Masks

mpf1t3 <- readTIFF('mpf1t3.tif')
mpf4t5 <- readTIFF('mpf4t5.tif')
mpfodl1 <- readTIFF('mpf-odl1.tif')

data.table.mask.type('Mid')
proPhoto.mmasks <-
  rbindlist(use.names=TRUE,
           list(
             mask.as.data.table.pf(mpf1t3[1,,1], '(1) Basic Mid-tones', y.scale = 3.984375),
             mask.as.data.table.pf(mpf1t3[1,,2], '(2) Expanded Mid-tones', y.scale = 1.770881),
             mask.as.data.table.pf(mpf1t3[1,,3], '(3) Wide Mid-tones', y.scale = 1.138413),
             mask.as.data.table.pf(mpf4t5[1,,1], '(4) Super Mid-tones', y.scale = 1.007905),
             mask.as.data.table.pf(mpf4t5[1,,2], '(5) Ultra Mid-tones', y.scale = 1)
#              ,mask.as.data.table.pf(mpfodl1[1,,1], '(6+) Inner Mid-tones', y.scale = 3.050893)
#              ,mask.as.data.table.pf(mpfodl1[1,,2], '(7+) Mid Mid-tones', y.scale = 1.723018)
           ))


x11()
plot.pf.mask(
  proPhoto.mmasks, mask.type='Mid-tones',
  show.lab.x = TRUE,
  plot.lab.x = TRUE,
  normal.scale = TRUE
)

### Zone Masks

zpf0t2 <- readTIFF('zpf0t2.tif')
zpf3t5 <- readTIFF('zpf3t5.tif')
zpf6t8 <- readTIFF('zpf6t8.tif')
zpf9tX <- readTIFF('zpf9tX.tif')

data.table.mask.type('Zone')
proPhoto.zmasks <-
  rbindlist(use.names=TRUE,
           list(
             mask.as.data.table.pf(zpf0t2[1,,1], 'Zone 0', y.scale = 1),
             mask.as.data.table.pf(zpf0t2[1,,2], 'Zone 1', y.scale = 1.608139),
             mask.as.data.table.pf(zpf0t2[1,,3], 'Zone 2', y.scale = 1.608139),
             mask.as.data.table.pf(zpf3t5[1,,1], 'Zone 3', y.scale = 1.608139),
             mask.as.data.table.pf(zpf3t5[1,,2], 'Zone 4', y.scale = 1.608139),
             mask.as.data.table.pf(zpf3t5[1,,3], 'Zone 5', y.scale = 2.462463),
             mask.as.data.table.pf(zpf6t8[1,,1], 'Zone 6', y.scale = 1.608139),
             mask.as.data.table.pf(zpf6t8[1,,2], 'Zone 7', y.scale = 1.608139),
             mask.as.data.table.pf(zpf6t8[1,,3], 'Zone 8', y.scale = 1.608139),
             mask.as.data.table.pf(zpf9tX[1,,1], 'Zone 9', y.scale = 1.608139),
             mask.as.data.table.pf(zpf9tX[1,,2], 'Zone X', y.scale = 1.608139)
           ))


x11()
plot.pf.mask(
  proPhoto.zmasks, mask.type='Zone',
  show.lab.x = FALSE,
  plot.lab.x = TRUE,
  normal.scale = TRUE
)

### Export proPhoto Data